@@ -21,12 +21,7 @@
 
 #include <libsoup/soup-message.h>
 #include <libsoup/soup-uri.h>
-#include "couchdb-database.h"
-#include "couchdb-design-document.h"
-#include "couchdb-document-info.h"
-#include "couchdb-document.h"
-#include "couchdb-marshal.h"
-#include "couchdb-types.h"
+#include "couchdb-glib.h"
 #include "dbwatch.h"
 #include "utils.h"
 
@@ -130,7 +125,6 @@ couchdb_database_class_init (CouchdbDatabaseClass *klass)
 	object_class->finalize = couchdb_database_finalize;
 	object_class->set_property = couchdb_database_set_property;
 	object_class->get_property = couchdb_database_get_property;
-	klass->create_document_from_json = NULL;
 
 	g_object_class_install_property (object_class,
         	                         PROP_SESSION,
@@ -154,9 +148,8 @@ couchdb_database_class_init (CouchdbDatabaseClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (CouchdbDatabaseClass, document_created),
 			      NULL, NULL,
-			      _couchdb_marshal_VOID__STRING_OBJECT,
-			      G_TYPE_NONE, 2,
-			      G_TYPE_STRING,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1,
 			      G_TYPE_OBJECT);
 	couchdb_database_signals[DOCUMENT_UPDATED] =
 		g_signal_new ("document_updated",
@@ -164,9 +157,8 @@ couchdb_database_class_init (CouchdbDatabaseClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (CouchdbDatabaseClass, document_updated),
 			      NULL, NULL,
-			      _couchdb_marshal_VOID__STRING_OBJECT,
-			      G_TYPE_NONE, 2,
-			      G_TYPE_STRING,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1,
 			      G_TYPE_OBJECT);
 	couchdb_database_signals[DOCUMENT_DELETED] =
 		g_signal_new ("document_deleted",
@@ -174,9 +166,8 @@ couchdb_database_class_init (CouchdbDatabaseClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (CouchdbDatabaseClass, document_deleted),
 			      NULL, NULL,
-			      _couchdb_marshal_VOID__STRING_STRING,
-			      G_TYPE_NONE, 2,
-			      G_TYPE_STRING,
+			      g_cclosure_marshal_VOID__STRING,
+			      G_TYPE_NONE, 1,
 			      G_TYPE_STRING);
 }
 
@@ -186,6 +177,35 @@ couchdb_database_init (CouchdbDatabase *database)
 	database->priv = g_new0 (CouchdbDatabasePrivate, 1);
 }
 
+static CouchdbDocument *
+create_document_from_json_object (JsonObject *json_object)
+{
+	CouchdbDocument *document;
+
+	if (g_str_has_prefix (json_object_get_string_member (json_object, "id"),
+			      "_design/")) {
+		document = COUCHDB_DOCUMENT (couchdb_design_document_new ());
+		couchdb_document_set_from_json_object (document,
+						       json_object_get_object_member (json_object, "doc"));
+	} else {
+		if (json_object_has_member (json_object, "record_type")) {
+			if (g_strcmp0 (json_object_get_string_member (json_object, "record_type"),
+				       COUCHDB_RECORD_TYPE_CONTACT) == 0) {
+				document = COUCHDB_DOCUMENT (couchdb_document_contact_new ());
+			} else if (g_strcmp0 (json_object_get_string_member (json_object, "record_type"),
+					      COUCHDB_RECORD_TYPE_TASK) == 0) {
+				document = COUCHDB_DOCUMENT (couchdb_document_task_new ());
+			} else
+				document = couchdb_document_new ();
+
+			couchdb_document_set_from_json_object (document, json_object);
+		} else
+			document = couchdb_document_new_from_json_object (json_object);
+	}
+
+	return document;
+}
+
 /**
  * couchdb_database_new:
  * @session: A #CouchdbSession object
@@ -328,7 +348,7 @@ couchdb_database_get_design_documents (CouchdbDatabase *database, GError **error
 
 			document = couchdb_design_document_new ();
 			if (document != NULL) {
-				couchdb_document_set_from_json_object (document, obj);
+				couchdb_document_set_from_json_object (COUCHDB_DOCUMENT (document), obj);
 				doclist = g_slist_append (doclist, document);
 			}
 		}
@@ -381,21 +401,7 @@ couchdb_database_get_all_documents (CouchdbDatabase *database, GError **error)
 			if (!obj)
 				continue;
 
-			if (g_str_has_prefix (json_object_get_string_member (obj, "id"),
-					      "_design/")) {
-				document = COUCHDB_DOCUMENT (couchdb_design_document_new ());
-				couchdb_document_set_from_json_object (document,
-								       json_object_get_object_member (obj, "doc"));
-			} else {
-				if (COUCHDB_DATABASE_GET_CLASS (database)->create_document_from_json != NULL) {
-					document = COUCHDB_DATABASE_GET_CLASS (database)->create_document_from_json (
-						database,
-						json_object_get_object_member (obj, "doc"));
-				} else {
-					document = couchdb_document_new_from_json_object (
-						json_object_get_object_member (obj, "doc"));
-				}
-			}
+			document = create_document_from_json_object (json_object_get_object_member (obj, "doc"));
 			if (document != NULL) {
 				g_object_set (G_OBJECT (document), "database", database, NULL);
 				doclist = g_slist_append (doclist, document);
@@ -466,25 +472,10 @@ couchdb_database_execute_view (CouchdbDatabase *database,
 			if (!obj)
 				continue;
 
-			if (g_str_has_prefix (json_object_get_string_member (obj, "id"),
-					      "_design/")) {
-				document = COUCHDB_DOCUMENT (couchdb_design_document_new ());
-				couchdb_document_set_from_json_object (document,
-								       json_object_get_object_member (obj, "value"));
-			} else {
-				if (COUCHDB_DATABASE_GET_CLASS (database)->create_document_from_json != NULL) {
-					document = COUCHDB_DATABASE_GET_CLASS (database)->create_document_from_json (
-						database,
-						json_object_get_object_member (obj, "value"));
-				} else {
-					document = couchdb_document_new_from_json_object (
-						json_object_get_object_member (obj, "value"));
-				}
-			}
-
-			couchdb_document_set_id (document, json_object_get_string_member (obj, "id"));
+			document = create_document_from_json_object (json_object_get_object_member (obj, "value"));
 
 			if (document != NULL) {
+				couchdb_document_set_id (document, json_object_get_string_member (obj, "id"));
 				g_object_set (G_OBJECT (document), "database", database, NULL);
 				doclist = g_slist_append (doclist, document);
 			}
@@ -550,13 +541,9 @@ couchdb_database_get_document (CouchdbDatabase *database,
 			       database->priv->dbname, encoded_docid);
 	parser = json_parser_new ();
 	if (couchdb_session_send_message (database->priv->session, SOUP_METHOD_GET, url, NULL, parser, error)) {
-		if (COUCHDB_DATABASE_GET_CLASS (database)->create_document_from_json != NULL) {
-			document = COUCHDB_DATABASE_GET_CLASS (database)->create_document_from_json (
-				database, json_node_get_object (json_parser_get_root (parser)));
-		} else {
-			document = couchdb_document_new_from_json_object (json_node_get_object (json_parser_get_root (parser)));
-		}
+		JsonObject *json_object = json_node_get_object (json_parser_get_root (parser));
 
+		document = create_document_from_json_object (json_object);
 		g_object_set (G_OBJECT (document), "database", database, NULL);
 	}
 
@@ -708,7 +695,6 @@ couchdb_database_delete_document (CouchdbDatabase *database, CouchdbDocument *do
 {
 	const char *id, *revision;
 	char *url;
-	JsonParser *parser;
 	gboolean result = FALSE;
 
 	g_return_val_if_fail (COUCHDB_IS_DATABASE (database), FALSE);
