@@ -22,278 +22,313 @@
 #include "couchdb-document-contact.h"
 #include "utils.h"
 
-CouchdbDocument *
-couchdb_document_contact_new (CouchdbSession *couchdb)
+G_DEFINE_TYPE(CouchdbDocumentContact, couchdb_document_contact, COUCHDB_TYPE_DOCUMENT)
+
+static void
+couchdb_document_contact_class_init (CouchdbDocumentContactClass *klass)
 {
-	CouchdbDocument *document;
-
-	g_return_val_if_fail (COUCHDB_IS_SESSION (couchdb), NULL);
-
-	document = couchdb_document_new (couchdb);
-	if (document)
-		couchdb_document_set_record_type (document, COUCHDB_RECORD_TYPE_CONTACT);
-
-	return document;
 }
 
-gboolean
-couchdb_document_is_contact (CouchdbDocument *document)
+static void
+couchdb_document_contact_init (CouchdbDocumentContact *document)
 {
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), FALSE);
+	couchdb_document_set_record_type (COUCHDB_DOCUMENT (document), COUCHDB_RECORD_TYPE_CONTACT);
+}
 
-	return !g_ascii_strcasecmp (couchdb_document_get_record_type (document),
-				    COUCHDB_RECORD_TYPE_CONTACT);
+/**
+ * couchdb_document_contact_new:
+ *
+ * Create a new #CouchdbDocumentContact object.
+ */
+CouchdbDocumentContact *
+couchdb_document_contact_new (void)
+{
+	return g_object_new (COUCHDB_TYPE_DOCUMENT_CONTACT, NULL);
 }
 
 const char *
-couchdb_document_contact_get_first_name (CouchdbDocument *document)
+couchdb_document_contact_get_title (CouchdbDocumentContact *document)
 {
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
 
-	return couchdb_document_get_string_field (document, "first_name");
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "title");
 }
 
 void
-couchdb_document_contact_set_first_name (CouchdbDocument *document, const char *first_name)
+couchdb_document_contact_set_title (CouchdbDocumentContact *document, const char *title)
 {
-	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
-	g_return_if_fail (couchdb_document_is_contact (document));
-	g_return_if_fail (first_name != NULL);
-
-	couchdb_document_set_string_field (document, "first_name", first_name);
-}
-
-const char *
-couchdb_document_contact_get_last_name (CouchdbDocument *document)
-{
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
-
-	return couchdb_document_get_string_field (document, "last_name");
-}
-
-void
-couchdb_document_contact_set_last_name (CouchdbDocument *document, const char *last_name)
-{
-      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
-	g_return_if_fail (couchdb_document_is_contact (document));
-	g_return_if_fail (last_name != NULL);
-
-	couchdb_document_set_string_field (document, "last_name", last_name);
-}
-
-const char *
-couchdb_document_contact_get_nick_name (CouchdbDocument *document)
-{
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
-
-	return couchdb_document_get_string_field (document, "nick_name");
-}
-
-void
-couchdb_document_contact_set_nick_name (CouchdbDocument *document, const char *nick_name)
-{
-      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
-	g_return_if_fail (couchdb_document_is_contact (document));
-	g_return_if_fail (nick_name != NULL);
-
-	couchdb_document_set_string_field (document, "nick_name", nick_name);
-}
-
-const char *
-couchdb_document_contact_get_spouse_name (CouchdbDocument *document)
-{
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
-
-	return couchdb_document_get_string_field (document, "spouse_name");
-}
-
-void
-couchdb_document_contact_set_spouse_name (CouchdbDocument *document, const char *spouse_name)
-{
-      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
-	g_return_if_fail (couchdb_document_is_contact (document));
-	g_return_if_fail (spouse_name != NULL);
-
-	couchdb_document_set_string_field (document, "spouse_name", spouse_name);
-}
-
-const char *
-couchdb_document_contact_get_birth_date (CouchdbDocument *document)
-{
-	JsonObject *object;
-
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
-
-	return couchdb_document_get_string_field (document, "birth_date");
-}
-
-void
-couchdb_document_contact_set_birth_date (CouchdbDocument *document, const char *birth_date)
-{
-	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
-	g_return_if_fail (couchdb_document_is_contact (document));
-	g_return_if_fail (birth_date != NULL);
-
-	couchdb_document_set_string_field (document, "birth_date", birth_date);
-}
-
-const char *
-couchdb_document_contact_get_wedding_date (CouchdbDocument *document)
-{
-	JsonObject *object;
-
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
-
-	return couchdb_document_get_string_field (document, "wedding_date");
-}
-
-void
-couchdb_document_contact_set_wedding_date (CouchdbDocument *document, const char *wedding_date)
-{
-	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
-	g_return_if_fail (couchdb_document_is_contact (document));
-	g_return_if_fail (wedding_date != NULL);
-
-	couchdb_document_set_string_field (document, "wedding_date", wedding_date);
-}
-
-const char *
-couchdb_document_contact_get_company (CouchdbDocument *document)
-{
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
-
-	return couchdb_document_get_string_field (document, "company");
-}
-
-void
-couchdb_document_contact_set_company (CouchdbDocument *document, const char *company)
-{
-      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
-	g_return_if_fail (couchdb_document_is_contact (document));
-	g_return_if_fail (company != NULL);
-
-	couchdb_document_set_string_field (document, "company", company);
-}
-
-const char *
-couchdb_document_contact_get_department (CouchdbDocument *document)
-{
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
-
-	return couchdb_document_get_string_field (document, "department");
-}
-
-void
-couchdb_document_contact_set_department (CouchdbDocument *document, const char *department)
-{
-      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
-	g_return_if_fail (couchdb_document_is_contact (document));
-	g_return_if_fail (department != NULL);
-
-	couchdb_document_set_string_field (document, "department", department);
-}
-
-const char *
-couchdb_document_contact_get_title (CouchdbDocument *document)
-{
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
-
-	return couchdb_document_get_string_field (document, "title");
-}
-
-void
-couchdb_document_contact_set_title (CouchdbDocument *document, const char *title)
-{
-      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
-	g_return_if_fail (couchdb_document_is_contact (document));
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
 	g_return_if_fail (title != NULL);
 
-	couchdb_document_set_string_field (document, "title", title);
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "title", title);
 }
 
 const char *
-couchdb_document_contact_get_job_title (CouchdbDocument *document)
+couchdb_document_contact_get_first_name (CouchdbDocumentContact *document)
 {
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
 
-	return couchdb_document_get_string_field (document, "job_title");
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "first_name");
 }
 
 void
-couchdb_document_contact_set_job_title (CouchdbDocument *document, const char *job_title)
+couchdb_document_contact_set_first_name (CouchdbDocumentContact *document, const char *first_name)
 {
-      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
-	g_return_if_fail (couchdb_document_is_contact (document));
+	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
+	g_return_if_fail (first_name != NULL);
+
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "first_name", first_name);
+}
+
+const char *
+couchdb_document_contact_get_middle_name (CouchdbDocumentContact *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
+
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "middle_name");
+}
+
+void
+couchdb_document_contact_set_middle_name (CouchdbDocumentContact *document, const char *middle_name)
+{
+	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
+	g_return_if_fail (middle_name != NULL);
+
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "middle_name", middle_name);
+}
+
+const char *
+couchdb_document_contact_get_last_name (CouchdbDocumentContact *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
+
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "last_name");
+}
+
+void
+couchdb_document_contact_set_last_name (CouchdbDocumentContact *document, const char *last_name)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
+	g_return_if_fail (last_name != NULL);
+
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "last_name", last_name);
+}
+
+const char *
+couchdb_document_contact_get_suffix (CouchdbDocumentContact *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
+
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "suffix");
+}
+
+void
+couchdb_document_contact_set_suffix (CouchdbDocumentContact *document, const char *suffix)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
+	g_return_if_fail (suffix != NULL);
+
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "suffix", suffix);
+}
+
+const char *
+couchdb_document_contact_get_nick_name (CouchdbDocumentContact *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
+
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "nick_name");
+}
+
+void
+couchdb_document_contact_set_nick_name (CouchdbDocumentContact *document, const char *nick_name)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
+	g_return_if_fail (nick_name != NULL);
+
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "nick_name", nick_name);
+}
+
+const char *
+couchdb_document_contact_get_spouse_name (CouchdbDocumentContact *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
+
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "spouse_name");
+}
+
+void
+couchdb_document_contact_set_spouse_name (CouchdbDocumentContact *document, const char *spouse_name)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
+	g_return_if_fail (spouse_name != NULL);
+
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "spouse_name", spouse_name);
+}
+
+const char *
+couchdb_document_contact_get_birth_date (CouchdbDocumentContact *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
+
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "birth_date");
+}
+
+void
+couchdb_document_contact_set_birth_date (CouchdbDocumentContact *document, const char *birth_date)
+{
+	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
+	g_return_if_fail (birth_date != NULL);
+
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "birth_date", birth_date);
+}
+
+const char *
+couchdb_document_contact_get_wedding_date (CouchdbDocumentContact *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
+
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "wedding_date");
+}
+
+void
+couchdb_document_contact_set_wedding_date (CouchdbDocumentContact *document, const char *wedding_date)
+{
+	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
+	g_return_if_fail (wedding_date != NULL);
+
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "wedding_date", wedding_date);
+}
+
+const char *
+couchdb_document_contact_get_company (CouchdbDocumentContact *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
+
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "company");
+}
+
+void
+couchdb_document_contact_set_company (CouchdbDocumentContact *document, const char *company)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
+	g_return_if_fail (company != NULL);
+
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "company", company);
+}
+
+const char *
+couchdb_document_contact_get_department (CouchdbDocumentContact *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
+
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "department");
+}
+
+void
+couchdb_document_contact_set_department (CouchdbDocumentContact *document, const char *department)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
+	g_return_if_fail (department != NULL);
+
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "department", department);
+}
+
+const char *
+couchdb_document_contact_get_job_title (CouchdbDocumentContact *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
+
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "job_title");
+}
+
+void
+couchdb_document_contact_set_job_title (CouchdbDocumentContact *document, const char *job_title)
+{
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
 	g_return_if_fail (job_title != NULL);
 
-	couchdb_document_set_string_field (document, "job_title", job_title);
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "job_title", job_title);
 }
 
 const char *
-couchdb_document_contact_get_manager_name (CouchdbDocument *document)
+couchdb_document_contact_get_manager_name (CouchdbDocumentContact *document)
 {
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
 
-	return couchdb_document_get_string_field (document, "manager_name");
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "manager_name");
 }
 
 void
-couchdb_document_contact_set_manager_name (CouchdbDocument *document, const char *manager_name)
+couchdb_document_contact_set_manager_name (CouchdbDocumentContact *document, const char *manager_name)
 {
-      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
-	g_return_if_fail (couchdb_document_is_contact (document));
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
 	g_return_if_fail (manager_name != NULL);
 
-	couchdb_document_set_string_field (document, "manager_name", manager_name);
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "manager_name", manager_name);
 }
 
 const char *
-couchdb_document_contact_get_assistant_name (CouchdbDocument *document)
+couchdb_document_contact_get_assistant_name (CouchdbDocumentContact *document)
 {
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
 
-	return couchdb_document_get_string_field (document, "assistant_name");
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "assistant_name");
 }
 
 void
-couchdb_document_contact_set_assistant_name (CouchdbDocument *document, const char *assistant_name)
+couchdb_document_contact_set_assistant_name (CouchdbDocumentContact *document, const char *assistant_name)
 {
-      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
-	g_return_if_fail (couchdb_document_is_contact (document));
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
 	g_return_if_fail (assistant_name != NULL);
 
-	couchdb_document_set_string_field (document, "assistant_name", assistant_name);
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "assistant_name", assistant_name);
 }
 
 const char *
-couchdb_document_contact_get_office (CouchdbDocument *document)
+couchdb_document_contact_get_office (CouchdbDocumentContact *document)
 {
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
 
-	return couchdb_document_get_string_field (document, "office");
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "office");
 }
 
 void
-couchdb_document_contact_set_office (CouchdbDocument *document, const char *office)
+couchdb_document_contact_set_office (CouchdbDocumentContact *document, const char *office)
 {
-      	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
-	g_return_if_fail (couchdb_document_is_contact (document));
+      	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
 	g_return_if_fail (office != NULL);
 
-	couchdb_document_set_string_field (document, "office", office);
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "office", office);
 }
 
 static void
@@ -316,7 +351,7 @@ foreach_object_cb (JsonObject *object,
 
 /**
  * couchdb_document_contact_get_email_addresses:
- * @document: A #CouchdbDocument object representing a contact
+ * @document: A #CouchdbDocumentContact object representing a contact
  *
  * Retrieve a list of email addresses from the specified contact document.
  * Email addresses are returned in a GSList of #CouchdbStructField objects,
@@ -328,16 +363,16 @@ foreach_object_cb (JsonObject *object,
  * Return value: a #GSList of #CouchdbStructField objects.
  */
 GSList *
-couchdb_document_contact_get_email_addresses (CouchdbDocument *document)
+couchdb_document_contact_get_email_addresses (CouchdbDocumentContact *document)
 {
 	GSList *list = NULL;
 	JsonObject *addresses_json;
 
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
 
 	addresses_json = json_object_get_object_member (
-		couchdb_document_get_json_object (document), "email_addresses");;
+		couchdb_document_get_json_object (COUCHDB_DOCUMENT (document)), "email_addresses");;
 	if (addresses_json) {
 		json_object_foreach_member (addresses_json,
 					    (JsonObjectForeach) foreach_object_cb,
@@ -348,12 +383,12 @@ couchdb_document_contact_get_email_addresses (CouchdbDocument *document)
 }
 
 void
-couchdb_document_contact_set_email_addresses (CouchdbDocument *document, GSList *list)
+couchdb_document_contact_set_email_addresses (CouchdbDocumentContact *document, GSList *list)
 {
 	GSList *l;
 	JsonObject *addresses_json;
 
-	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
 
 	addresses_json = json_object_new ();
 	for (l = list; l != NULL; l = l->next) {
@@ -375,20 +410,22 @@ couchdb_document_contact_set_email_addresses (CouchdbDocument *document, GSList
 		}
 	}
 
-	json_object_set_object_member (couchdb_document_get_json_object (document), "email_addresses", addresses_json);
+	json_object_set_object_member (couchdb_document_get_json_object (COUCHDB_DOCUMENT (document)),
+				       "email_addresses",
+				       addresses_json);
 }
 
 GSList *
-couchdb_document_contact_get_phone_numbers (CouchdbDocument *document)
+couchdb_document_contact_get_phone_numbers (CouchdbDocumentContact *document)
 {
 	GSList *list = NULL;
 	JsonObject *phone_numbers;
 
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
 
 	phone_numbers = json_object_get_object_member (
-		couchdb_document_get_json_object (document), "phone_numbers");
+		couchdb_document_get_json_object (COUCHDB_DOCUMENT (document)), "phone_numbers");
 	if (phone_numbers) {
 		json_object_foreach_member (phone_numbers,
 					    (JsonObjectForeach) foreach_object_cb,
@@ -399,12 +436,12 @@ couchdb_document_contact_get_phone_numbers (CouchdbDocument *document)
 }
 
 void
-couchdb_document_contact_set_phone_numbers (CouchdbDocument *document, GSList *list)
+couchdb_document_contact_set_phone_numbers (CouchdbDocumentContact *document, GSList *list)
 {
 	GSList *l;
 	JsonObject *phone_numbers;
 
-	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
 
 	phone_numbers = json_object_new ();
 	for (l = list; l != NULL; l = l->next) {
@@ -428,20 +465,21 @@ couchdb_document_contact_set_phone_numbers (CouchdbDocument *document, GSList *l
 		}
 	}
 
-	json_object_set_object_member (couchdb_document_get_json_object (document), "phone_numbers", phone_numbers);
+	json_object_set_object_member (couchdb_document_get_json_object (COUCHDB_DOCUMENT (document)),
+				       "phone_numbers", phone_numbers);
 }
 
 GSList *
-couchdb_document_contact_get_addresses (CouchdbDocument *document)
+couchdb_document_contact_get_addresses (CouchdbDocumentContact *document)
 {
 	GSList *list = NULL;
 	JsonObject *addresses;
 
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
 
 	addresses = json_object_get_object_member (
-		couchdb_document_get_json_object (document), "addresses");
+		couchdb_document_get_json_object (COUCHDB_DOCUMENT (document)), "addresses");
 	if (addresses) {
 		json_object_foreach_member (addresses,
 					    (JsonObjectForeach) foreach_object_cb,
@@ -452,12 +490,12 @@ couchdb_document_contact_get_addresses (CouchdbDocument *document)
 }
 
 void
-couchdb_document_contact_set_addresses (CouchdbDocument *document, GSList *list)
+couchdb_document_contact_set_addresses (CouchdbDocumentContact *document, GSList *list)
 {
 	GSList *l;
 	JsonObject *addresses;
 
-	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
 
 	addresses = json_object_new ();
 	for (l = list; l != NULL; l = l->next) {
@@ -469,7 +507,10 @@ couchdb_document_contact_set_addresses (CouchdbDocument *document, GSList *list)
 			JsonObject *this_address;
 
 			this_address = json_object_new ();
-			json_object_set_string_member (this_address, "street", street_str);
+
+			json_object_set_string_member (this_address, "address1", street_str);
+			json_object_set_string_member (this_address, "address2",
+						       couchdb_document_contact_address_get_ext_street (sf));
 			json_object_set_string_member (this_address, "city",
 						       couchdb_document_contact_address_get_city (sf));
 			json_object_set_string_member (this_address, "state",
@@ -489,20 +530,21 @@ couchdb_document_contact_set_addresses (CouchdbDocument *document, GSList *list)
 		}
 	}
 
-	json_object_set_object_member (couchdb_document_get_json_object (document), "addresses", addresses);
+	json_object_set_object_member (couchdb_document_get_json_object (COUCHDB_DOCUMENT (document)),
+				       "addresses", addresses);
 }
 
 GSList *
-couchdb_document_contact_get_im_addresses (CouchdbDocument *document)
+couchdb_document_contact_get_im_addresses (CouchdbDocumentContact *document)
 {
 	GSList *list = NULL;
 	JsonObject *im_addresses;
 
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
 
 	im_addresses = json_object_get_object_member (
-		couchdb_document_get_json_object (document), "im_addresses");
+		couchdb_document_get_json_object (COUCHDB_DOCUMENT (document)), "im_addresses");
 	if (im_addresses != NULL) {
 		json_object_foreach_member (im_addresses,
 					    (JsonObjectForeach) foreach_object_cb,
@@ -513,12 +555,12 @@ couchdb_document_contact_get_im_addresses (CouchdbDocument *document)
 }
 
 void
-couchdb_document_contact_set_im_addresses (CouchdbDocument *document, GSList *list)
+couchdb_document_contact_set_im_addresses (CouchdbDocumentContact *document, GSList *list)
 {
 	GSList *l;
 	JsonObject *im_addresses_json;
 
-	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
 
 	im_addresses_json = json_object_new ();
 	for (l = list; l != NULL; l = l->next) {
@@ -542,20 +584,21 @@ couchdb_document_contact_set_im_addresses (CouchdbDocument *document, GSList *li
 		}
 	}
 
-	json_object_set_object_member (couchdb_document_get_json_object (document), "im_addresses", im_addresses_json);
+	json_object_set_object_member (couchdb_document_get_json_object (COUCHDB_DOCUMENT (document)),
+				       "im_addresses", im_addresses_json);
 }
 
 GSList *
-couchdb_document_contact_get_urls (CouchdbDocument *document)
+couchdb_document_contact_get_urls (CouchdbDocumentContact *document)
 {
 	GSList *list = NULL;
 	JsonObject *urls;
 
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
 
 	urls = json_object_get_object_member (
-		couchdb_document_get_json_object (document), "urls");
+		couchdb_document_get_json_object (COUCHDB_DOCUMENT (document)), "urls");
 	if (urls) {
 		json_object_foreach_member (urls,
 					    (JsonObjectForeach) foreach_object_cb,
@@ -566,12 +609,12 @@ couchdb_document_contact_get_urls (CouchdbDocument *document)
 }
 
 void
-couchdb_document_contact_set_urls (CouchdbDocument *document, GSList *list)
+couchdb_document_contact_set_urls (CouchdbDocumentContact *document, GSList *list)
 {
 	GSList *l;
 	JsonObject *urls_json;
 
-	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
 
 	urls_json = json_object_new ();
 	for (l = list; l != NULL; l = l->next) {
@@ -593,26 +636,54 @@ couchdb_document_contact_set_urls (CouchdbDocument *document, GSList *list)
 		}
 	}
 
-	json_object_set_object_member (couchdb_document_get_json_object (document), "urls", urls_json);
+	json_object_set_object_member (couchdb_document_get_json_object (COUCHDB_DOCUMENT (document)),
+				       "urls", urls_json);
 }
 
+/**
+ * couchdb_document_contact_get_categories:
+ * @document: A #CouchdbDocumentContact object
+ *
+ * Get the list of categories (as a string) for this contact document.
+ *
+ * Return value: A comma separated list of categories as a string.
+ */
 const char *
-couchdb_document_contact_get_notes (CouchdbDocument *document)
+couchdb_document_contact_get_categories (CouchdbDocumentContact *document)
 {
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (couchdb_document_is_contact (document), NULL);
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
 
-	return couchdb_document_get_string_field (document, "notes");
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "categories");
 }
 
 void
-couchdb_document_contact_set_notes (CouchdbDocument *document, const char *notes)
+couchdb_document_contact_set_categories (CouchdbDocumentContact *document, const char *categories)
 {
-	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
-	g_return_if_fail (couchdb_document_is_contact (document));
+	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
+	g_return_if_fail (categories != NULL);
+
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "categories", categories);
+}
+
+const char *
+couchdb_document_contact_get_notes (CouchdbDocumentContact *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document), NULL);
+	g_return_val_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)), NULL);
+
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "notes");
+}
+
+void
+couchdb_document_contact_set_notes (CouchdbDocumentContact *document, const char *notes)
+{
+	g_return_if_fail (COUCHDB_IS_DOCUMENT_CONTACT (document));
+	g_return_if_fail (couchdb_document_is_contact (COUCHDB_DOCUMENT (document)));
 	g_return_if_fail (notes != NULL);
 
-	couchdb_document_set_string_field (document, "notes", notes);
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "notes", notes);
 }
 
 CouchdbStructField *
@@ -744,13 +815,14 @@ couchdb_document_contact_phone_set_description (CouchdbStructField *sf, const ch
 
 CouchdbStructField *
 couchdb_document_contact_address_new (const char *uuid,
-				      const char *street,
-				      const char *city,
-				      const char *state,
-				      const char *country,
-				      const char *postalcode,
-				      const char *pobox,
-				      const char *description)
+					   const char *street,
+					   const char *ext_street,
+					   const char *city,
+					   const char *state,
+					   const char *country,
+					   const char *postalcode,
+					   const char *pobox,
+					   const char *description)
 {
 	CouchdbStructField *sf;
 
@@ -765,6 +837,8 @@ couchdb_document_contact_address_new (const char *uuid,
 
 	if (street)
 		couchdb_document_contact_address_set_street (sf, street);
+	if (ext_street)
+		couchdb_document_contact_address_set_ext_street (sf, ext_street);
 	if (city)
 		couchdb_document_contact_address_set_city (sf, city);
 	if (state)
@@ -786,7 +860,12 @@ couchdb_document_contact_address_get_street (CouchdbStructField *sf)
 {
 	g_return_val_if_fail (sf != NULL, NULL);
 
-	return couchdb_struct_field_get_string_field (sf, "street");
+	if (couchdb_struct_field_has_field (sf, "address1"))
+		return couchdb_struct_field_get_string_field (sf, "address1");
+	else if (couchdb_struct_field_has_field (sf, "street"))
+		return couchdb_struct_field_get_string_field (sf, "street");
+
+	return NULL;
 }
 
 void
@@ -794,7 +873,23 @@ couchdb_document_contact_address_set_street (CouchdbStructField *sf, const char
 {
 	g_return_if_fail (sf != NULL);
 
-	couchdb_struct_field_set_string_field (sf, "street", street);
+	couchdb_struct_field_set_string_field (sf, "address1", street);
+}
+
+const char *
+couchdb_document_contact_address_get_ext_street (CouchdbStructField *sf)
+{
+	g_return_val_if_fail (sf != NULL, NULL);
+
+	return couchdb_struct_field_get_string_field (sf, "address2");
+}
+
+void
+couchdb_document_contact_address_set_ext_street (CouchdbStructField *sf, const char *ext_street)
+{
+	g_return_if_fail (sf != NULL);
+
+	couchdb_struct_field_set_string_field (sf, "address2", ext_street);
 }
 
 const char *
