@@ -117,6 +117,7 @@ static void
 couchdb_document_init (CouchdbDocument *document)
 {
 	document->priv = g_new0 (CouchdbDocumentPrivate, 1);
+	document->priv->root_object = json_object_new ();
 }
 
 /**
@@ -134,8 +135,6 @@ couchdb_document_new (void)
 
 	document = g_object_new (COUCHDB_TYPE_DOCUMENT, NULL);
 
-	document->priv->root_object = json_object_new ();
-
 	return document;
 }
 
@@ -146,6 +145,9 @@ couchdb_document_new_from_json_object (JsonObject *json_object)
 
 	document = g_object_new (COUCHDB_TYPE_DOCUMENT, NULL);
 
+	if (document->priv->root_object != NULL)
+		json_object_unref (document->priv->root_object);
+
 	document->priv->root_object = json_object_ref (json_object);
 
 	return document;
@@ -266,8 +268,6 @@ couchdb_document_set_revision (CouchdbDocument *document, const char *revision)
 gboolean
 couchdb_document_has_field (CouchdbDocument *document, const char *field)
 {
-	JsonNode *node;
-
 	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), FALSE);
 	g_return_val_if_fail (field != NULL, FALSE);
 
@@ -552,7 +552,7 @@ couchdb_document_foreach_attachment (CouchdbDocument *document,
 {
 	CouchdbStructField *attachments;
 
-	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
 
 	attachments = couchdb_document_get_struct_field (document, "_attachments");
 	if (attachments != NULL) {
@@ -610,3 +610,111 @@ couchdb_document_get_json_object (CouchdbDocument *document)
 	
 	return document->priv->root_object;
 }
+
+/**
+ * couchdb_document_get_record_type:
+ * @document: A #CouchdbDocument object
+ *
+ * Retrieve the record type of the given document. Record types are special
+ * fields in the CouchDB documents, used in Desktopcouch, to identify
+ * standard records. All supported record types are listed at
+ * http://www.freedesktop.org/wiki/Specifications/desktopcouch#Formats.
+ *
+ * Return value: The record type of the given document.
+ */
+const char *
+couchdb_document_get_record_type (CouchdbDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+
+	return couchdb_document_get_string_field (COUCHDB_DOCUMENT (document), "record_type");
+}
+
+/**
+ * couchdb_document_set_record_type:
+ * @document: A #CouchdbDocument object
+ * @record_type: Record type to set the document to
+ *
+ * Set the record type of the given document.
+ */
+void
+couchdb_document_set_record_type (CouchdbDocument *document, const char *record_type)
+{
+	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+	g_return_if_fail (record_type != NULL);
+
+	couchdb_document_set_string_field (COUCHDB_DOCUMENT (document), "record_type", record_type);
+}
+
+/**
+ * couchdb_document_get_application_annotations:
+ * @document: A #CouchdbDocument object
+ *
+ * Retrieve the application annotations for the given document.
+ *
+ * Application annotations is a special field (named "application_annotations"), used
+ * in Desktopcouch to allow applications to set values on standard documents (as defined
+ * at http://www.freedesktop.org/wiki/Specifications/desktopcouch#Formats) that are
+ * not part of the standard, but still needed by the application.
+ *
+ * Return value: A #CouchdbStructField containing the value of the application
+ * annotations for the given document.
+ */
+CouchdbStructField *
+couchdb_document_get_application_annotations (CouchdbDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), NULL);
+
+	return couchdb_document_get_struct_field (COUCHDB_DOCUMENT (document), "application_annotations");
+}
+
+/**
+ * couchdb_document_set_application_annotations:
+ * @document: A #CouchdbDocument object
+ * @annotations: A #CouchdbStructField with the contents of the application_annotations field.
+ *
+ * Set the application annotations for the given document.
+ */
+void
+couchdb_document_set_application_annotations (CouchdbDocument *document, CouchdbStructField *annotations)
+{
+	g_return_if_fail (COUCHDB_IS_DOCUMENT (document));
+
+	couchdb_document_set_struct_field (COUCHDB_DOCUMENT (document), "application_annotations", annotations);
+}
+
+/**
+ * couchdb_document_is_contact:
+ * @document: A #CouchdbDocument object
+ *
+ * Check whether the given document represents a contact record, as specified
+ * at http://www.freedesktop.org/wiki/Specifications/desktopcouch/contact
+ *
+ * Return value: TRUE if the document represents a contact, FALSE otherwise.
+ */
+gboolean
+couchdb_document_is_contact (CouchdbDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), FALSE);
+
+	return !g_ascii_strcasecmp (couchdb_document_get_record_type (document),
+				    COUCHDB_RECORD_TYPE_CONTACT);
+}
+
+/**
+ * couchdb_document_is_task:
+ * @document: A #CouchdbDocument object
+ *
+ * Check whether the given document represents a contact record, as specified
+ * at http://www.freedesktop.org/wiki/Specifications/desktopcouch/task
+ *
+ * Return value: TRUE if the document represents a task, FALSE otherwise.
+ */
+gboolean
+couchdb_document_is_task (CouchdbDocument *document)
+{
+	g_return_val_if_fail (COUCHDB_IS_DOCUMENT (document), FALSE);
+
+	return !g_ascii_strcasecmp (couchdb_document_get_record_type (document),
+				    COUCHDB_RECORD_TYPE_TASK);
+}
