@@ -32,17 +32,26 @@
 
 G_BEGIN_DECLS
 
-#define COUCHDB_TYPE_STRUCT_FIELD  (couchdb_struct_field_get_type ())
+#define COUCHDB_TYPE_STRUCT_FIELD                (couchdb_struct_field_get_type ())
+#define COUCHDB_STRUCT_FIELD(obj)                (G_TYPE_CHECK_INSTANCE_CAST ((obj), COUCHDB_TYPE_STRUCT_FIELD, CouchdbStructField))
+#define COUCHDB_IS_STRUCT_FIELD(obj)             (G_TYPE_CHECK_INSTANCE_TYPE ((obj), COUCHDB_TYPE_STRUCT_FIELD))
+#define COUCHDB_STRUCT_FIELD_CLASS(klass)        (G_TYPE_CHECK_CLASS_CAST ((klass), COUCHDB_TYPE_STRUCT_FIELD, CouchdbStructFieldClass))
+#define COUCHDB_IS_STRUCT_FIELD_CLASS(klass)     (G_TYPE_CHECK_CLASS_TYPE ((klass), COUCHDB_TYPE_STRUCT_FIELD))
+#define COUCHDB_STRUCT_FIELD_GET_CLASS(obj)      (G_TYPE_INSTANCE_GET_CLASS ((obj), COUCHDB_TYPE_STRUCT_FIELD, CouchdbStructFieldClass))
+
+typedef struct {
+	GObjectClass parent_class;
+} CouchdbStructFieldClass;
 
 GType               couchdb_struct_field_get_type (void);
 CouchdbStructField *couchdb_struct_field_new (void);
 CouchdbStructField *couchdb_struct_field_new_from_string (const char *str);
-CouchdbStructField *couchdb_struct_field_ref (CouchdbStructField *sf);
-void                couchdb_struct_field_unref (CouchdbStructField *sf);
 
 gboolean            couchdb_struct_field_has_field (CouchdbStructField *sf, const char *field);
 void                couchdb_struct_field_remove_field (CouchdbStructField *sf, const char *field);
 
+GSList             *couchdb_struct_field_get_field_names (CouchdbStructField *sf);
+
 CouchdbArrayField  *couchdb_struct_field_get_array_field (CouchdbStructField *sf, const char *field);
 void                couchdb_struct_field_set_array_field (CouchdbStructField *sf, const char *field, CouchdbArrayField *value);
 gboolean            couchdb_struct_field_get_boolean_field (CouchdbStructField *sf, const char *field);
